==> mule-standalone-3.1.0.zip: org/mule/util/queue/TransactionalQueueSession.java <==
/*
 * $Id: TransactionalQueueSession.java 19191 2010-08-25 21:05:23Z tcarlson $
 * --------------------------------------------------------------------------------------
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 *
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */

package org.mule.util.queue;

import org.mule.util.xa.AbstractXAResourceManager;
import org.mule.util.xa.DefaultXASession;

import java.io.IOException;

/**
 * A Queue session that is used to manage the transaction context of a Queue
 */
class TransactionalQueueSession extends DefaultXASession implements QueueSession
{

    protected TransactionalQueueManager queueManager;

    public TransactionalQueueSession(AbstractXAResourceManager resourceManager,
                                     TransactionalQueueManager queueManager)
    {
        super(resourceManager);
        this.queueManager = queueManager;
    }

    public Queue getQueue(String name)
    {
        QueueInfo queue = queueManager.getQueue(name);
        return new QueueImpl(queue);
    }

    protected class QueueImpl implements Queue
    {

        protected QueueInfo queue;

        public QueueImpl(QueueInfo queue)
        {
            this.queue = queue;
        }

        public void put(Object item) throws InterruptedException
        {
            offer(item, Long.MAX_VALUE);
        }

        public boolean offer(Object item, long timeout) throws InterruptedException
        {
            if (localContext != null)
            {
                return ((TransactionalQueueManager.QueueTransactionContext) localContext).offer(queue, item,
                    timeout);
            }
            else
            {
                try
                {
                    Object id = queueManager.doStore(queue, item);
                    try
                    {
                        if (!queue.offer(id, 0, timeout))
                        {
                            queueManager.doRemove(queue, id);
                            return false;
                        }
                        else
                        {
                            return true;
                        }
                    }
                    catch (InterruptedException e)
                    {
                        queueManager.doRemove(queue, id);
                        throw e;
                    }
                }
                catch (IOException e)
                {
                    throw new RuntimeException(e);
                }
            }
        }

        public Object take() throws InterruptedException
        {
            return poll(Long.MAX_VALUE);
        }

        public void untake(Object item) throws InterruptedException
        {
            queue.untake(item);
        }

        public Object poll(long timeout) throws InterruptedException
        {
            try
            {
                if (localContext != null)
                {
                    return ((TransactionalQueueManager.QueueTransactionContext) localContext).poll(queue,
                                                                                                   timeout);
                }
                else
                {
                    Object id = queue.poll(timeout);
                    if (id != null)
                    {
                        Object item = queueManager.doLoad(queue, id);
                        queueManager.doRemove(queue, id);
                        return item;
                    }
                    return null;
                }
            }
            catch (InterruptedException iex)
            {
                if (queueManager.getMuleContext().isStopping())
                {
                    throw iex;
                }
                // if stopping, ignore
                return null;
            }
            catch (IOException e)
            {
                throw new RuntimeException(e);
            }

        }

        public Object peek() throws InterruptedException
        {
            try
            {
                if (localContext != null)
                {
                    return ((TransactionalQueueManager.QueueTransactionContext) localContext).peek(queue);
                }
                else
                {
                    Object id = queue.peek();
                    if (id != null)
                    {
                        return queueManager.doLoad(queue, id);
                    }
                    return null;
                }
            }
            catch (IOException e)
            {
                throw new RuntimeException(e);
            }
        }

        public int size()
        {
            if (localContext != null)
            {
                return ((TransactionalQueueManager.QueueTransactionContext) localContext).size(queue);
            }
            else
            {
                return queue.list.size();
            }
        }

        public String getName()
        {
            return queue.getName();
        }

    }
}

==> mule-standalone-3.1.1.zip: org/mule/util/queue/TransactionalQueueSession.java <==
/*
 * $Id: TransactionalQueueSession.java 21210 2011-02-09 13:46:01Z dfeist $
 * --------------------------------------------------------------------------------------
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 *
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */

package org.mule.util.queue;

import org.mule.util.xa.AbstractXAResourceManager;
import org.mule.util.xa.DefaultXASession;

import java.io.IOException;

/**
 * A Queue session that is used to manage the transaction context of a Queue
 */
class TransactionalQueueSession extends DefaultXASession implements QueueSession
{

    protected TransactionalQueueManager queueManager;

    public TransactionalQueueSession(AbstractXAResourceManager resourceManager,
                                     TransactionalQueueManager queueManager)
    {
        super(resourceManager);
        this.queueManager = queueManager;
    }

    public Queue getQueue(String name)
    {
        QueueInfo queue = queueManager.getQueue(name);
        return new QueueImpl(queue);
    }

    protected class QueueImpl implements Queue
    {

        protected QueueInfo queue;

        public QueueImpl(QueueInfo queue)
        {
            this.queue = queue;
        }

        public void put(Object item) throws InterruptedException
        {
            offer(item, Long.MAX_VALUE);
        }

        public boolean offer(Object item, long timeout) throws InterruptedException
        {
            if (localContext != null)
            {
                return ((TransactionalQueueManager.QueueTransactionContext) localContext).offer(queue, item,
                    timeout);
            }
            else
            {
                try
                {
                    Object id = queueManager.doStore(queue, item);
                    try
                    {
                        if (!queue.offer(id, 0, timeout))
                        {
                            queueManager.doRemove(queue, id);
                            return false;
                        }
                        else
                        {
                            return true;
                        }
                    }
                    catch (InterruptedException e)
                    {
                        queueManager.doRemove(queue, id);
                        throw e;
                    }
                }
                catch (IOException e)
                {
                    throw new RuntimeException(e);
                }
            }
        }

        public Object take() throws InterruptedException
        {
            return poll(Long.MAX_VALUE);
        }

        public void untake(Object item) throws InterruptedException
        {
            if (localContext != null)
            {
                ((TransactionalQueueManager.QueueTransactionContext) localContext).untake(queue, item);
            }
            else
            {
                try
                {
                    Object id = queueManager.doStore(queue, item);
                    queue.untake(id);
                }
                catch (IOException e)
                {
                    throw new RuntimeException(e);
                }
            }
        }

        public Object poll(long timeout) throws InterruptedException
        {
            try
            {
                if (localContext != null)
                {
                    return ((TransactionalQueueManager.QueueTransactionContext) localContext).poll(queue,
                                                                                                   timeout);
                }
                else
                {
                    Object id = queue.poll(timeout);
                    if (id != null)
                    {
                        Object item = queueManager.doLoad(queue, id);
                        queueManager.doRemove(queue, id);
                        return item;
                    }
                    return null;
                }
            }
            catch (InterruptedException iex)
            {
                if (queueManager.getMuleContext().isStopping())
                {
                    throw iex;
                }
                // if stopping, ignore
                return null;
            }
            catch (IOException e)
            {
                throw new RuntimeException(e);
            }

        }

        public Object peek() throws InterruptedException
        {
            try
            {
                if (localContext != null)
                {
                    return ((TransactionalQueueManager.QueueTransactionContext) localContext).peek(queue);
                }
                else
                {
                    Object id = queue.peek();
                    if (id != null)
                    {
                        return queueManager.doLoad(queue, id);
                    }
                    return null;
                }
            }
            catch (IOException e)
            {
                throw new RuntimeException(e);
            }
        }

        public int size()
        {
            if (localContext != null)
            {
                return ((TransactionalQueueManager.QueueTransactionContext) localContext).size(queue);
            }
            else
            {
                return queue.list.size();
            }
        }

        public String getName()
        {
            return queue.getName();
        }

    }
}
